==> Botania-1.19.2-438-FABRIC-20230214.034242-11-sources.jar: vazkii/botania/common/item/equipment/bauble/RingOfDexterousMotionItem.java <==
/*
 * This class is distributed as part of the Botania Mod.
 * Get the Source Code in github:
 * https://github.com/Vazkii/Botania
 *
 * Botania is Open Source and distributed under the
 * Botania License: http://botaniamod.net/license.php
 */
package vazkii.botania.common.item.equipment.bauble;

import com.mojang.blaze3d.systems.RenderSystem;
import net.minecraft.class_1309;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_2350;
import net.minecraft.class_243;
import net.minecraft.class_310;
import net.minecraft.class_332;
import net.minecraft.class_3532;
import net.minecraft.class_4587;
import vazkii.botania.client.core.handler.ClientTickHandler;
import vazkii.botania.common.handler.EquipmentHandler;
import vazkii.botania.common.helper.ItemNBTHelper;
import vazkii.botania.common.item.BotaniaItems;
import vazkii.botania.network.serverbound.DodgePacket;
import vazkii.botania.xplat.ClientXplatAbstractions;

public class RingOfDexterousMotionItem extends BaubleItem {

	public static final String TAG_DODGE_COOLDOWN = "dodgeCooldown";
	public static final int MAX_CD = 20;

	private static boolean oldLeftDown, oldRightDown, oldForwardDown, oldBackDown;
	private static int leftDown, rightDown, forwardDown, backDown;

	public RingOfDexterousMotionItem(class_1793 props) {
		super(props);
	}

	public static class ClientLogic {
		public static void onKeyDown() {
			class_310 mc = class_310.method_1551();
			if (mc.field_1724 == null) {
				return;
			}

			class_1799 ringStack = EquipmentHandler.findOrEmpty(BotaniaItems.dodgeRing, mc.field_1724);
			if (ringStack.method_7960() || ItemNBTHelper.getInt(ringStack, TAG_DODGE_COOLDOWN, 0) > 0) {
				return;
			}

			int threshold = 5;
			if (mc.field_1690.field_1913.method_1434() && !oldLeftDown) {
				int oldLeft = leftDown;
				leftDown = ClientTickHandler.ticksInGame;

				if (leftDown - oldLeft < threshold) {
					dodge(mc.field_1724, class_2350.field_11039);
				}
			} else if (mc.field_1690.field_1849.method_1434() && !oldRightDown) {
				int oldRight = rightDown;
				rightDown = ClientTickHandler.ticksInGame;

				if (rightDown - oldRight < threshold) {
					dodge(mc.field_1724, class_2350.field_11034);
				}
			} else if (mc.field_1690.field_1894.method_1434() && !oldForwardDown) {
				int oldForward = forwardDown;
				forwardDown = ClientTickHandler.ticksInGame;

				if (forwardDown - oldForward < threshold) {
					dodge(mc.field_1724, class_2350.field_11043);
				}
			} else if (mc.field_1690.field_1881.method_1434() && !oldBackDown) {
				int oldBack = backDown;
				backDown = ClientTickHandler.ticksInGame;

				if (backDown - oldBack < threshold) {
					dodge(mc.field_1724, class_2350.field_11035);
				}
			}

			oldLeftDown = mc.field_1690.field_1913.method_1434();
			oldRightDown = mc.field_1690.field_1849.method_1434();
			oldForwardDown = mc.field_1690.field_1894.method_1434();
			oldBackDown = mc.field_1690.field_1881.method_1434();
		}

		public static void renderHUD(class_4587 ms, class_1657 player, class_1799 stack, float pticks) {
			int xo = class_310.method_1551().method_22683().method_4486() / 2 - 20;
			int y = class_310.method_1551().method_22683().method_4502() / 2 + 20;

			if (!player.method_31549().field_7479) {
				int cd = ItemNBTHelper.getInt(stack, TAG_DODGE_COOLDOWN, 0);
				int width = Math.min((int) ((cd - pticks) * 2), 40);
				RenderSystem.setShaderColor(1F, 1F, 1F, 1F);
				if (width > 0) {
					class_332.method_25294(ms, xo, y - 2, xo + 40, y - 1, 0x88000000);
					class_332.method_25294(ms, xo, y - 2, xo + width, y - 1, 0xFFFFFFFF);
				}
			}

			RenderSystem.setShaderColor(1F, 1F, 1F, 1F);
		}
	}

	@Override
	public void onWornTick(class_1799 stack, class_1309 living) {
		int cd = ItemNBTHelper.getInt(stack, TAG_DODGE_COOLDOWN, 0);
		if (cd > 0) {
			ItemNBTHelper.setInt(stack, TAG_DODGE_COOLDOWN, cd - 1);
		}
	}

	private static void dodge(class_1657 player, class_2350 dir) {
		if (player.method_31549().field_7479 || !player.method_24828() || dir == class_2350.field_11036 || dir == class_2350.field_11033) {
			return;
		}

		float yaw = player.method_36454();
		float x = class_3532.method_15374(-yaw * 0.017453292F - (float) Math.PI);
		float z = class_3532.method_15362(-yaw * 0.017453292F - (float) Math.PI);
		if (dir == class_2350.field_11043 || dir == class_2350.field_11035) {
			x = class_3532.method_15362(-yaw * 0.017453292F);
			z = class_3532.method_15374(yaw * 0.017453292F);
		}
		class_243 lookVec = new class_243(x, 0, z);
		class_243 sideVec = lookVec.method_1036(new class_243(0, dir == class_2350.field_11039 || dir == class_2350.field_11043 ? 1 : (dir == class_2350.field_11034 || dir == class_2350.field_11035 ? -1 : 0), 0)).method_1021(1.25);

		player.method_18799(sideVec);

		ClientXplatAbstractions.INSTANCE.sendToServer(DodgePacket.INSTANCE);
	}
}

==> Botania-1.20.1-451-FABRIC-20260116.182542-39-sources.jar: vazkii/botania/common/item/equipment/bauble/RingOfDexterousMotionItem.java <==
/*
 * This class is distributed as part of the Botania Mod.
 * Get the Source Code in github:
 * https://github.com/Vazkii/Botania
 *
 * Botania is Open Source and distributed under the
 * Botania License: http://botaniamod.net/license.php
 */
package vazkii.botania.common.item.equipment.bauble;

import com.mojang.blaze3d.systems.RenderSystem;
import net.minecraft.class_1309;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_2350;
import net.minecraft.class_243;
import net.minecraft.class_310;
import net.minecraft.class_332;
import net.minecraft.class_3532;
import vazkii.botania.client.core.handler.ClientTickHandler;
import vazkii.botania.common.handler.EquipmentHandler;
import vazkii.botania.common.helper.ItemNBTHelper;
import vazkii.botania.common.item.BotaniaItems;
import vazkii.botania.network.serverbound.DodgePacket;
import vazkii.botania.xplat.ClientXplatAbstractions;

public class RingOfDexterousMotionItem extends BaubleItem {

	public static final String TAG_DODGE_COOLDOWN = "dodgeCooldown";
	public static final int MAX_CD = 20;

	private static boolean oldLeftDown, oldRightDown, oldForwardDown, oldBackDown;
	private static int leftDown, rightDown, forwardDown, backDown;

	public RingOfDexterousMotionItem(class_1793 props) {
		super(props);
	}

	public static class ClientLogic {
		public static void onKeyDown() {
			class_310 mc = class_310.method_1551();
			if (mc.field_1724 == null) {
				return;
			}

			class_1799 ringStack = EquipmentHandler.findOrEmpty(BotaniaItems.dodgeRing, mc.field_1724);
			if (ringStack.method_7960() || ItemNBTHelper.getInt(ringStack, TAG_DODGE_COOLDOWN, 0) > 0) {
				return;
			}

			int threshold = 5;
			if (mc.field_1690.field_1913.method_1434() && !oldLeftDown) {
				int oldLeft = leftDown;
				leftDown = ClientTickHandler.ticksInGame;

				if (leftDown - oldLeft < threshold) {
					dodge(mc.field_1724, class_2350.field_11039);
				}
			} else if (mc.field_1690.field_1849.method_1434() && !oldRightDown) {
				int oldRight = rightDown;
				rightDown = ClientTickHandler.ticksInGame;

				if (rightDown - oldRight < threshold) {
					dodge(mc.field_1724, class_2350.field_11034);
				}
			} else if (mc.field_1690.field_1894.method_1434() && !oldForwardDown) {
				int oldForward = forwardDown;
				forwardDown = ClientTickHandler.ticksInGame;

				if (forwardDown - oldForward < threshold) {
					dodge(mc.field_1724, class_2350.field_11043);
				}
			} else if (mc.field_1690.field_1881.method_1434() && !oldBackDown) {
				int oldBack = backDown;
				backDown = ClientTickHandler.ticksInGame;

				if (backDown - oldBack < threshold) {
					dodge(mc.field_1724, class_2350.field_11035);
				}
			}

			oldLeftDown = mc.field_1690.field_1913.method_1434();
			oldRightDown = mc.field_1690.field_1849.method_1434();
			oldForwardDown = mc.field_1690.field_1894.method_1434();
			oldBackDown = mc.field_1690.field_1881.method_1434();
		}

		public static void renderHUD(class_332 gui, class_1657 player, class_1799 stack, float pticks) {
			int xo = class_310.method_1551().method_22683().method_4486() / 2 - 20;
			int y = class_310.method_1551().method_22683().method_4502() / 2 + 20;

			if (!player.method_31549().field_7479) {
				int cd = ItemNBTHelper.getInt(stack, TAG_DODGE_COOLDOWN, 0);
				int width = Math.min((int) ((cd - pticks) * 2), 40);
				RenderSystem.setShaderColor(1F, 1F, 1F, 1F);
				if (width > 0) {
					gui.method_25294(xo, y - 2, xo + 40, y - 1, 0x88000000);
					gui.method_25294(xo, y - 2, xo + width, y - 1, 0xFFFFFFFF);
				}
			}

			RenderSystem.setShaderColor(1F, 1F, 1F, 1F);
		}
	}

	@Override
	public void onWornTick(class_1799 stack, class_1309 living) {
		int cd = ItemNBTHelper.getInt(stack, TAG_DODGE_COOLDOWN, 0);
		if (cd > 0) {
			ItemNBTHelper.setInt(stack, TAG_DODGE_COOLDOWN, cd - 1);
		}
	}

	private static void dodge(class_1657 player, class_2350 dir) {
		if (player.method_31549().field_7479 || !player.method_24828() || dir == class_2350.field_11036 || dir == class_2350.field_11033) {
			return;
		}

		float yaw = player.method_36454();
		float x = class_3532.method_15374(-yaw * 0.017453292F - (float) Math.PI);
		float z = class_3532.method_15362(-yaw * 0.017453292F - (float) Math.PI);
		if (dir == class_2350.field_11043 || dir == class_2350.field_11035) {
			x = class_3532.method_15362(-yaw * 0.017453292F);
			z = class_3532.method_15374(yaw * 0.017453292F);
		}
		class_243 lookVec = new class_243(x, 0, z);
		class_243 sideVec = lookVec.method_1036(new class_243(0, dir == class_2350.field_11039 || dir == class_2350.field_11043 ? 1 : (dir == class_2350.field_11034 || dir == class_2350.field_11035 ? -1 : 0), 0)).method_1021(1.25);

		player.method_18799(sideVec);

		ClientXplatAbstractions.INSTANCE.sendToServer(DodgePacket.INSTANCE);
	}
}
